using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;

using Syntriax.Engine.Core.Abstract;

namespace Syntriax.Engine.Core;

public static class BehaviourExtensions
{
    public static T? FindBehaviour<T>(this IEnumerable<IHierarchyObject> hierarchyObjects) where T : class
    {
        foreach (IHierarchyObject hierarchyObject in hierarchyObjects)
            if (hierarchyObject.BehaviourController.GetBehaviour<T>() is T behaviour)
                return behaviour;

        return default;
    }

    public static bool TryFindBehaviour<T>(this IEnumerable<IHierarchyObject> hierarchyObjects, [NotNullWhen(returnValue: true)] out T? behaviour) where T : class
    {
        behaviour = FindBehaviour<T>(hierarchyObjects);
        return behaviour is not null;
    }

    public static void FindBehaviours<T>(this IEnumerable<IHierarchyObject> hierarchyObjects, List<T> behaviours) where T : class
    {
        behaviours.Clear();
        List<T> cache = [];

        foreach (IHierarchyObject hierarchyObject in hierarchyObjects)
        {
            hierarchyObject.BehaviourController.GetBehaviours(cache);
            behaviours.AddRange(cache);
        }
    }
}
